/////////////////////////////////////////////////////////////////////
//
// OWLAVIDL.CPP Version 1.0
//
// OWL 2.0 Dll to play an AVI video in an About dialog box
//
// Author: Kent Reisdorph, 75522,1174
// Date: May 5, 1995
// Usage: TAviDlg(TWindow* parent,
//						TModule* mod,
//						const char far* title,
//						const char far* version,
//						const char far* date,
//						const char far* filename);
//
//	Copyright  1995 by Kent Reisdorph, All rights reserved
//
/////////////////////////////////////////////////////////////////////

#include <owl\dialog.h>
#include <owl\static.h>
#include <string.h>
#include <mmsystem.h>
#pragma hdrstop

#include "owlavidg.rh"

#define AVI_DLL
#include "owlavidg.h"


// The TModule*, LibMain, and WEP are provided in case you want
// to specifically use the dll instance for loading resources

//static TModule* DllModule;

//int FAR PASCAL LibMain (HINSTANCE hInstance, WORD, WORD, LPSTR)
//{
// use the following code to get a TModule* to this dll
//	if (DllModule == 0)
//   {
//		DllModule = new TModule(0, hInstance);
//   }
//	return TRUE;
//}

//int FAR PASCAL WEP (int)
//{
	// if you use the TModule* then delete it on unloading the dll
	// delete DllModule;
//	return TRUE;
//}

DEFINE_RESPONSE_TABLE1(TAviDlg, TDialog)
	EV_MESSAGE(MM_MCINOTIFY, MciNotify),
	EV_COMMAND(IDOK, CmOk),
END_RESPONSE_TABLE;

// Constructor. Sets up the controls and copies the passed char
// strings to class variables. Also sets an abritrary Right and
// Bottom margin. Change these settings if you don't like the look.
//
// Note that the TModule* does not default to 0. You must pass a
// module pointer to this class.
//
// NOTE: You can use any dialog template you wish. It must however
// have the four controls listed below unless you change the code.
// The three text statics are obvious. The IDC_AVIWINDOW static is
// the window that holds the AVI video. The static is automatically
// sized based on the size of the video image. The upper left corner
// of this static is the anchor point for the image.
TAviDlg::TAviDlg(TWindow* parent, TModule* mod,
						const char* far title,
						const char* far version,
						const char* far date,
						const char* far filename) :
	TDialog(parent, "AboutBox", mod)
{
	strcpy(Title, title);
	strcpy(Ver, version);
	strcpy(Date, date);
	strcpy(Filename, filename);
	stTitle = new TStatic(this, IDC_TITLE, 0, mod);
	stVersion = new TStatic(this, IDC_VERSION, 0, mod);
	stDate = new TStatic(this, IDC_DATE, 0, mod);
	AviWindow = new TStatic(this, IDC_AVIWINDOW, 0, mod);
	Button = new TButton(this, IDOK, mod);
	DeviceOpen = 0;
	RightMargin = GetSystemMetrics(SM_CYCAPTION) * 1.5;
	BottomMargin = RightMargin * 2;
}

// SetupWindow() Sets the text in the static controls and the title.
// The ugly code following is the code to resize the AVI static,
// resize the dialog based on the size of the video window, and to
// move the OK button to the center of the dialog.
void
TAviDlg::SetupWindow()
{
	TDialog::SetupWindow();
	stTitle->SetText(Title);
	stVersion->SetText(Ver);
	stDate->SetText(Date);
	char str[50] = "About ";
	strcat(str, Title);
	SetCaption(str);
	BOOL Bad;
	DWORD ErrCode = OpenAvi();
	Bad = CheckError(ErrCode);
	if (Bad) return;
	if (!Bad) ErrCode = PlayAvi();
	TRect vdRect = GetVideoRect();
	TRect stRect = AviWindow->GetWindowRect();
	TRect dgRect = GetWindowRect();
	TPoint UlPoint = stRect.TopLeft();
	int x = UlPoint.x + vdRect.Width() + RightMargin;
	int y = UlPoint.y + vdRect.Height() + BottomMargin;
	TPoint dgPoint = dgRect.TopLeft();
	dgRect.Set(dgPoint.x, dgPoint.y, x, y);
	MoveWindow(dgRect);
	stRect = Button->GetWindowRect();
	x = dgRect.Width()/2;
	x = x - (stRect.Width()/2);
	int z = GetSystemMetrics(SM_CYCAPTION) + (GetSystemMetrics(SM_CYBORDER) * 2);
	y = dgRect.Height() - (stRect.Height() + 20 + z);
	Button->MoveWindow(x, y, stRect.Width(), stRect.Height());
	ScreenToClient(UlPoint);
	AviWindow->MoveWindow(
		UlPoint.x, UlPoint.y, vdRect.Width(), vdRect.Height());
}

// CmOk() All done. Close the device before leaving.
void
TAviDlg::CmOk()
{
	mciSendCommand(DeviceID, MCI_CLOSE, 0, NULL);
	TDialog::CmOk();
}

// MciNotify() This function gets called when the MCI_NOTIFY message
// is sent. This message gets sent when the video reaches the end,
// that is, completes playing. If the video has reached the end then
// we Rewind() and Play() again.
LRESULT
TAviDlg::MciNotify(WPARAM, LPARAM)
{
	MCI_STATUS_PARMS Status;
	Status.dwItem = MCI_STATUS_MODE;
	mciSendCommand(DeviceID, MCI_STATUS,
		MCI_STATUS_ITEM, (DWORD)(LPVOID) &Status);
	if (Status.dwReturn == MCI_MODE_STOP) {
		RewindAvi();
		PlayAvi();
	}
	return 0;
}

// OpenAvi() The proper video driver is loaded based on the filename
// extension. Returns 0 on success or an error code if an error occured.
DWORD
TAviDlg::OpenAvi()
{
	DWORD flags =
		MCI_OVLY_OPEN_PARENT | MCI_OVLY_OPEN_WS |
		MCI_OPEN_ELEMENT;
	UINT devID;
	MCI_OVLY_OPEN_PARMS Open;
	//Open.lpstrDeviceType = "avivideo";
	Open.lpstrElementName = Filename;
	Open.dwStyle = WS_CHILD | WS_VISIBLE;
	Open.hWndParent = GetDlgItem(IDC_AVIWINDOW);
	if (DeviceOpen) {
		devID = DeviceID;
	}
	else devID = 0;
	DWORD ErrCode = mciSendCommand(devID,
		MCI_OPEN, flags, (DWORD)(LPVOID) &Open);
	if (!ErrCode) {
		DeviceOpen = TRUE;
		DeviceID = Open.wDeviceID;
	}
	return ErrCode;
}

// PlayAvi() No surprise here.
DWORD
TAviDlg::PlayAvi()
{
	DWORD flags = MCI_NOTIFY;
	MCI_PLAY_PARMS Play;
	Play.dwCallback = MAKELONG(HWindow,0);
	DWORD ErrCode = mciSendCommand(DeviceID,
		MCI_PLAY, flags, (DWORD)(LPVOID) &Play);
	return ErrCode;
}

// GetVideoRect() Returns a rect containing the size of the AVI
// video rectangle.
RECT
TAviDlg::GetVideoRect()
{
	MCI_OVLY_RECT_PARMS Parms;
	DWORD flags = MCI_WAIT | MCI_OVLY_WHERE_SOURCE;
	mciSendCommand(DeviceID, MCI_WHERE, flags, (DWORD)(LPVOID) &Parms);
	return Parms.rc;
}

// RewindAvi() Uses the MCI_SEEK command to position the video
// at the beginning.
void
TAviDlg::RewindAvi()
{
	MCI_SEEK_PARMS Seek;
	mciSendCommand(DeviceID, MCI_SEEK,
		MCI_SEEK_TO_START, (DWORD)(LPVOID) &Seek);
}

// CheckError() Check the error code if an error occurred on opening
// the Mci driver.
BOOL
TAviDlg::CheckError(DWORD errCode)
{
	switch (errCode) {
		case MCIERR_DEVICE_NOT_INSTALLED :
		case MCIERR_CANNOT_LOAD_DRIVER :
		case MCIERR_FILE_NOT_FOUND :
		case MCIERR_OUT_OF_MEMORY :
		if (DeviceOpen) mciSendCommand(DeviceID, MCI_CLOSE, 0, NULL);
		DeviceOpen = FALSE;
// the following three statements are for diagnostics. use them for
// debugging but you may want to take them out of the final product
//		char ErrMsg[256];
//		mciGetErrorString(ErrCode, ErrMsg, sizeof(ErrMsg));
//		MessageBox(0, ErrMsg, "Error", MB_OK);
		return TRUE;
	}
	return FALSE;
}
